package cn.com.qmth.examcloud.core.oe.student.face.controller;

import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.StudentCacheBean;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.UrlUtil;
import cn.com.qmth.examcloud.core.basic.api.FaceCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentFaceReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentFaceResp;
import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamCaptureEntity;
import cn.com.qmth.examcloud.core.oe.student.face.controller.bean.GetExamCaptureResultDomain;
import cn.com.qmth.examcloud.core.oe.student.face.service.ExamCaptureService;
import cn.com.qmth.examcloud.core.oe.student.face.service.bean.CompareFaceSyncInfo;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.upyun.UpYunHttpRequest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;


/**
 * @author chenken
 * @date 2018年9月6日 上午10:14:23
 * @company QMTH
 * @description ExamCaptureController.java
 */
@Api(tags = "考试抓拍")
@RestController
@RequestMapping("${app.api.oe.student.face}/examCaptureQueue")
public class ExamCaptureController extends ControllerSupport {

    @Autowired
    private ExamCaptureService examCaptureService;

    @Autowired
    private FaceCloudService faceCloudService;
    @Autowired
    RedisClient redisClient;

//	private static AES aes = new AES();

    @ApiOperation(value = "同步比较人脸:用于进入考试")
    @PostMapping("/compareFaceSync")
    public CompareFaceSyncInfo compareFaceSync(@RequestParam String fileUrl, @RequestParam(required = false) String signIdentifier) {
        User user = getAccessUser();
        Check.isBlank(fileUrl, "文件Url不能为空");
        validateUpyunSign(signIdentifier, fileUrl, user.getUserId());

        StudentCacheBean studentCache = CacheHelper.getStudent(user.getUserId());
        String baseFaceToken = studentCache.getFaceToken();
        Check.isBlank(baseFaceToken, "学生底照的faceToken为空");
//		fileUrl = aes.decrypt(fileUrl);
        fileUrl = UrlUtil.decode(fileUrl);
        return examCaptureService.compareFaceSyncByFileUrl(user.getUserId(), baseFaceToken, fileUrl);
    }

    /**
     * 校验又拍云签名
     *
     * @param signIndentifier 签名标识
     * @param fileUrl         文件路径
     * @param userId          用户id
     */
    private void validateUpyunSign(String signIndentifier, String fileUrl, Long userId) {
//		if (StringUtils.isBlank(signIndentifier)){
//			throw new StatusException("300001", "签名标识不能为空");
//		}
        if (!StringUtils.isBlank(signIndentifier)) {
            String upyunSignRedisKey = Constants.EXAM_CAPTURE_PHOTO_UPYUN_SIGN_PREFIX + userId + "_" + signIndentifier;
            UpYunHttpRequest upYunHttpRequest = redisClient.get(upyunSignRedisKey, UpYunHttpRequest.class);
            if (upYunHttpRequest == null) {
                throw new StatusException("300002", "无效的请求，请检查签名标识");
            }
            if (!upYunHttpRequest.getAccessUrl().equals(fileUrl)) {
                throw new StatusException("300003", "文件路径格式不正确");
            }
        }

    }

    @ApiOperation(value = "获取抓拍结果")
    @GetMapping("/getExamCaptureResult")
    public GetExamCaptureResultDomain getExamCaptureResult(@RequestParam Long examRecordDataId, @RequestParam String fileName) {
        Check.isNull(examRecordDataId, "examRecordDataId不能为空");
        Check.isBlank(fileName, "fileName不能为空");
        ExamCaptureEntity examCaptureEntity = examCaptureService.getExamCaptureResult(examRecordDataId, fileName);
        GetExamCaptureResultDomain domain = new GetExamCaptureResultDomain();
        if (examCaptureEntity == null) {
            domain.setIsCompleted(false);
        } else {
            domain.setIsCompleted(true);
            domain.setIsPass(examCaptureEntity.getIsPass());
            domain.setIsStranger(examCaptureEntity.getIsStranger());
            domain.setExamRecordDataId(examRecordDataId);
            domain.setFileName(fileName);
        }
        return domain;
    }
}
