package cn.com.qmth.examcloud.core.oe.student.face.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.concurrency.simple.Worker;
import cn.com.qmth.examcloud.commons.helpers.concurrency.simple.WorkerController;
import cn.com.qmth.examcloud.commons.util.Util;
import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamCaptureQueueStatus;
import cn.com.qmth.examcloud.core.oe.student.face.service.ExamCaptureQueueService;
import cn.com.qmth.examcloud.core.oe.student.face.service.ExamCaptureService;
import cn.com.qmth.examcloud.core.oe.student.face.service.bean.ExamCaptureQueueInfo;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.logging.log4j.ThreadContext;

import java.util.concurrent.TimeUnit;

/**
 * face++人脸比对工作线程
 *
 * @author lideyin 20190620
 */
public class FacePPCompareWorker implements Worker<ExamCaptureQueueInfo> {
    private final Log captureLog = LogFactory.getLog("PROCESS_EXAM_CAPTURE_TASK_LOGGER");

    @Override
    public void process(WorkerController controller, ExamCaptureQueueInfo element) {

        ThreadContext.put("TRACE_ID", "Q_" + element.getId());
        ThreadContext.put("CALLER", "FACEPP_WORKER");
        // 图片处理数量+1
        ExamCaptureProcessStatisticController.increaseFaceCompareCount();
        if (captureLog.isDebugEnabled()) {
            captureLog.debug("[FACEPP_COMPARE_WORKER.] 图片处理数量+1，count= "
                    + ExamCaptureProcessStatisticController.getFaceCompareCount());
        }
        ExamCaptureService examCaptureService = SpringContextHolder
                .getBean(ExamCaptureService.class);
        ExamCaptureQueueService examCaptureQueueService = SpringContextHolder
                .getBean(ExamCaptureQueueService.class);
        try {
            examCaptureService.disposeFaceCompare(element);
        } catch (StatusException e) {
            while (true){
                // 异常处理
                if(examCaptureQueueService.saveExamCaptureQueueEntityByFailed(element.getId(),e.getDesc(),
                        ExamCaptureQueueStatus.PROCESS_FACE_COMPARE_FAILED)){
                    break;
                };
                Util.sleep(TimeUnit.MILLISECONDS,500);
            }

            if ((e.getCode().equals(Constants.FACE_COMPARE_CONCURRENCY_LIMIT_EXCEEDED))) {
                // 如果超过并发次数，则添加异常次数
                controller.addConcurrencyWarn();
                captureLog.error("[FACEPP_COMPARE_WORKER.] 超过并发次数 ");
            } else {
                ExamCaptureProcessStatisticController.increaseFaceCompareFailedCount();
                captureLog.error("[FACEPP_COMPARE_WORKER.] 自定义异常 " +
                        ",failedCount=" + ExamCaptureProcessStatisticController.getFaceCompareFailedCount(), e);
            }

            Util.sleep(TimeUnit.MICROSECONDS, 50);
        } catch (Exception e) {
            ExamCaptureProcessStatisticController.increaseFaceCompareFailedCount();
            // 异常处理
            while (true){
                // 异常处理
                if(examCaptureQueueService.saveExamCaptureQueueEntityByFailed(element.getId(),"系统异常",
                        ExamCaptureQueueStatus.PROCESS_FACE_COMPARE_FAILED)){
                    break;
                }
                Util.sleep(TimeUnit.MILLISECONDS,500);
            }
            captureLog.error("[FACEPP_COMPARE_WORKER.] 系统异常 "  +
                    ",failedCount=" + ExamCaptureProcessStatisticController.getFaceCompareFailedCount(), e);
            Util.sleep(TimeUnit.MICROSECONDS, 50);
        } finally {
            ThreadContext.clearAll();
        }
    }
}
